const version = require('../package.json').version;
const path = require('path');
const fs = require('fs');

function renderFile(filePath, data, isComponent = false) {
    let content = fs.readFileSync(filePath);
    let template = content.toString();
    let head = "let str = '';with(obj){str+=`",
    tail = "`; return str;}";
    
    const variable = /\{\{([^}]+)\}\}/g;
    const expression = /\{\%([^%]+)\%\}/g;

    template = head + template;
    template = template.replace(variable, (_, r) => "${" + r + "}");
    template = template.replace(expression, (_, r) => "`;" + r + ";str+=`");
    template += tail;

    try {
        const fn = new Function("obj", template);
        return (!isComponent ? (
            '<!-- \n' +
            '    rendered by render.javascript@' + version + ' \n' +
            '    npmjs.com/package/render.javascript \n' +
            '-->'
        ) : '') + fn(Object.assign(data, {
            include: _path => {
                let parts = filePath.split('/');
                let file = parts.slice(0, parts.length - 1).join('/');
                return renderFile(file + '/' + _path, data, true);
            }
        })).replace('\r\n', '');
    } catch(parseErr) {
        parseErr.name = 'Render.js Error';
        const fileContent = fs.readFileSync(filePath).toString();
        console.error(parseErr);
        
        return fs.readFileSync(path.join(__dirname, './error.html')).toString()
            .replace(/{{error}}/g, parseErr.name + ': ' + parseErr.message)
            .replace(/{{code}}/g, fileContent)
            .replace(/{{stack}}/g, parseErr.stack?.trimStart());
    };
};


class RenderJS {
    /**
     * @param {String} template Write the text you want to be rendered.
     */
    constructor(template) {
        this.template = template || '';
        return this;
    };

    /**
     * @param {Object} data Write the data to be used in the render!
     * @returns Rendered HTML Page
     */
    render(data) {
        if (typeof this.template == 'string') {
            return this.template.replace(/{{(.*?)}}/g, match => {
                return data[match.split(/{{|}}/).filter(Boolean)[0].trim()];
            });
        } else {
            throw new Error('Render.js Error: Template must be a string!');
        };
    };
};

module.exports = RenderJS;
module.exports.renderFile = (filePath, data, cb) => cb(null, renderFile(filePath, data));
module.exports.version = version;